<template>
  <div class="mod-config">
    <basic-container>
      <el-row>
        <el-col :span="13">
          <ttable
            :table="table"
            @currentChange="handleCurrentChange"
            :uploadInfo="uploadInfo"
            :prelang="prelang"
            :options="options"
            :ajaxFun="ajaxFun"
            ref="operationTaskTable"
          >
            <template #toolbar>
              <el-button
                v-if="permissions.operationtaskreserved_pick_scm"
                @click="pickScm()"
                type="primary"
                style="margin-left:10px;"
                >SCM领料
              </el-button>
            </template>
          </ttable>
        </el-col>
        <el-col :span="11" style="padding-left: 10px">
          <ttable
            :table="joinTable"
            @handleSelectionChange="joinHandleSelectionChange"
            :uploadInfo="joinUploadInfo"
            :prelang="joinPrelang"
            :options="joinOptions"
            :ajaxFun="joinAjaxFun"
            :paramObj="paramObj"
            ref="joinStockTable"
          >
            <template #toolbar></template>
          </ttable>
        </el-col>
      </el-row>

      <!-- 预留库存零件 -->
      <!--<stockDialog
        :currshowlist.sync="relateVisible"
        @handleSelectionChange="saveJoinData"
        :paramSelArr="paramSelArr"
        :paramSelCol="paramSelCol"
        :multiSelect="stockMultiSelect"
        :showIsUsed="false"
        :partNo="partNo"
        :paramArr="paramArr"
      />-->
      <!-- 修改预留库存数量 -->
      <joinStockOrderDialog v-if="addOrUpdateVisible" ref="addOrUpdate" />
      <!-- 预留 -->
      <reservedOperateForm
        :currshowlist.sync="relateVisible"
        :taskId="taskId"
        @handleReserved="handleReserved"
      />
    </basic-container>
  </div>
</template>

<script>
import { mapGetters } from 'vuex'
import ttable from '@/views/common/ztt-table.vue'
import stockDialog from '@/views/plan/operationtaskreserved/stock'
import joinStockOrderDialog from './join-stock-order'
import reservedOperateForm from '@/views/plan/operationtaskreserved/reserved-operate-form'
import { fetchList as fetchListOperation } from '@/api/plan/operationtask'
import {
  addObj,
  changeState,
  delByIds,
  cancelReserve,
  fetchList
} from '@/api/warehouse/joinstockorder'
import { updateOperationTaskId } from '../../../api/warehouse/joinstockorder'
import { matPick } from '@/api/scm/common'

export default {
  data() {
    return {
      ajaxFun: fetchListOperation,
      currentCustomerOrder: null,
      currentOperationTask: null,
      taskId: null,
      uploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      prelang: 'operation',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: false, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮H
        isSearch: true, // 高级查询按钮
        defaultOrderBy: { column: 'plannedStartDate', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            minWidth: '140',
            prop: 'optaskNo',
            label: '工单编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            propVal: this.$route.query.optaskNo
          },
          {
            minWidth: '140',
            prop: 'moNo',
            label: '制造订单编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'partName',
            label: '零件',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'partNo',
            label: '零件号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'operationName',
            label: '工序',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'priority',
            label: '优先级',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'state',
            label: '状态',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text',
            formatter: (row, column, cellValue) => {
              var formatVal
              if (cellValue == '01pending') {
                formatVal = '等待'
              } else if (cellValue == '02inProgress') {
                formatVal = '进行中'
              } else if (cellValue == '03interrupted') {
                formatVal = '已暂停'
              } else if (cellValue == '04completed') {
                formatVal = '已完成'
              } else if (cellValue == '05canceled') {
                formatVal = '已取消'
              } else {
                formatVal = ''
              }
              return formatVal
            }
          },
          {
            minWidth: '120',
            prop: 'plannedQuantity',
            label: '计划数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'completedQuantity',
            label: '完成数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '20',
            prop: 'unit',
            label: '单位',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'createTime',
            label: '创建时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'plannedStartDate',
            label: '计划开始时间',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'plannedFinishDate',
            label: '计划完成时间',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'actualStartDate',
            label: '实际完成时间',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'actualFinishDate',
            label: '实际完成时间',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'remark',
            label: '备注',
            sort: false,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text'
          }
        ],
        toolbar: [],
        operator: null,
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 200,
          minWidth: 100
        }
      },
      coStateList: [
        {
          value: '01plan',
          label: '待计划'
        },
        {
          value: '03planed',
          label: '已计划'
        },
        {
          value: '02planing',
          label: '计划中'
        },
        {
          value: '04complete',
          label: '已完成'
        }
      ],
      partNo: '',

      paramObj: { operationTaskId: 0 },
      paramArr: [],
      joinAjaxFun: fetchList,
      joinMultipleSelection: [],
      joinUploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      joinPrelang: 'operation',
      joinOptions: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮
        isSearch: false, // 高级查询按钮
        defaultOrderBy: { column: 'id', direction: 'asc' }
      },
      joinTable: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            minWidth: '100',
            prop: 'status',
            label: '状态',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatStatus,
            optList: () => {
              return this.statusOptions
            }
          },
          {
            minWidth: '110',
            prop: 'partNo',
            label: '零件编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '110',
            prop: 'partName',
            label: '零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'warehouseName',
            label: '仓库',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'locationNo',
            label: '库位号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'partBatchNo',
            label: 'SN号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'reelNumber',
            label: '载具编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '110',
            prop: 'reservedQuantity',
            label: '预留数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'number'
          },
          {
            minWidth: '110',
            prop: 'deliveryQuantity',
            label: '发货数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'number'
          },
          {
            minWidth: '100',
            prop: 'partUnit',
            label: '单位',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          }
        ],
        toolbar: [
          {
            text: '预留',
            type: 'primary',
            fun: this.reservedHandle,
            disabled: false
          },
          {
            text: '取消预留',
            type: 'primary',
            fun: this.cancelReservedHandle,
            disabled: false
          },
          {
            text: 'ERP取消预留',
            type: 'primary',
            fun: this.cancelReservedHandle1,
            disabled: false
          }
        ],
        operator: null,
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 60,
          minWidth: 60
        }
      },
      addOrUpdateVisible: false,
      statusOptions: [
        {
          value: '02confirmed',
          label: '已确认',
          colorLabel: '已确认'
        }
      ],
      paramSelArr: [],
      paramSelCol: 'id',
      stockMultiSelect: false,
      relateVisible: false
    }
  },
  components: {
    ttable,
    stockDialog,
    joinStockOrderDialog,
    reservedOperateForm
  },
  computed: {
    ...mapGetters(['permissions'])
  },
  watch: {},
  methods: {
    getData() {
      this.$refs.customerOrderTable.getDataList()
    },
    handleCurrentChange(row) {
      // this.joinTable.toolbar.forEach((item) => {
      //   item.disabled = true
      // })
      if (!row) {
        this.currentOperationTask = null
        this.paramObj = { operationTaskId: 0 }
        this.partNo = ''
      } else if (
        !this.currentOperationTask ||
        this.currentOperationTask.id !== row.id
      ) {
        this.currentOperationTask = row
        this.paramObj = { operationTaskId: row.id }
        // if (row.coState === '03planed' || row.coState === '02planing') {
        //   this.joinTable.toolbar.forEach((item) => {
        //     item.disabled = false
        //   })
        // }
      }
      this.$nextTick(() => {
        this.getJoinData()
      })
    },

    getJoinData() {
      this.$refs.joinStockTable.getDataList()
    },
    joinHandleSelectionChange(val) {
      this.joinMultipleSelection = val
    },
    formatStatus(row, column, cellValue) {
      this.statusOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.colorLabel
        }
      })
      return cellValue
    },
    // 修改预留数量
    addOrUpdateHandle(row) {
      this.addOrUpdateVisible = true
      this.$nextTick(() => {
        this.$refs.addOrUpdate.init(row)
      })
    },
    // 打开预留库存选择页面
    reservedHandle() {
      if (this.currentOperationTask) {
        this.taskId = this.currentOperationTask.id
        this.relateVisible = true
      } else {
        this.$message.warning('请先选择一条工单')
      }
    },
    // 打开预留库存选择页面
    /* reservedHandle() {
      if (this.currentOperationTask) {
        // 筛选出当前已经预留的库存
        this.paramSelArr = []
        fetchAll(this.paramObj).then((response) => {
          if (response.data.data) {
            response.data.data.forEach((e) => {
              this.paramSelArr.push(e.stockId)
            })
          }
          this.paramArr = [this.currentOperationTask.id]
          this.relateVisible = true
          this.stockMultiSelect = true
        })
      } else {
        this.$message.warning('请先选择一条工单')
      }
    }, */
    // 保存预留库存
    saveJoinData(stockList, isUsed) {
      if (this.currentOperationTask && stockList && stockList.length > 0) {
        const joinStockOrderList = []
        stockList.forEach((item) => {
          // 可用库存大于0才能预留
          if (item.availableStockQuantity > 0) {
            joinStockOrderList.push(
              Object.assign({
                operationTaskId: this.currentOperationTask.id,
                stockId: item.id,
                reservedQuantity: item.availableStockQuantity,
                partNo: item.partNo,
                partName: item.partName,
                warehouseNo: item.warehouseNo,
                warehouseName: item.warehouseName,
                locationNo: item.locationNo,
                locationName: item.locationName,
                partUnit: item.unit,
                systemNo: item.systemNo,
                partBatchNo: item.partBatchNo,
                status: '01unconfirmed',
                type: '工单预留'
              })
            )
          }
        })
        if (joinStockOrderList && joinStockOrderList.length > 0) {
          addObj(joinStockOrderList).then((response) => {
            this.$refs.joinStockTable.getDataList()
            this.$message.success('预留成功')
          })
        }
      }
    },
    // 取消预留
    cancelReservedHandle() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        const joinStockOrderList = []
        let joinMultiple
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          joinMultiple = this.joinMultipleSelection[i]
          joinStockOrderList.push({
            id: joinMultiple.id,
            operationTaskId: joinMultiple.operationTaskId,
            stockId: joinMultiple.stockId,
            reservedQuantity: joinMultiple.reservedQuantity,
            partNo: joinMultiple.partNo,
            partName: joinMultiple.partName,
            warehouseNo: joinMultiple.warehouseNo,
            warehouseName: joinMultiple.warehouseName,
            locationNo: joinMultiple.locationNo,
            locationName: joinMultiple.locationName,
            partUnit: joinMultiple.partUnit,
            systemNo: joinMultiple.systemNo,
            partBatchNo: joinMultiple.partBatchNo,
            status: '02confirmed',
            type: '工单预留'
          })
        }
        cancelReserve(joinStockOrderList)
          .then((response) => {
            const resCode = response.data.code
            if (resCode === 0) {
              this.$refs.joinStockTable.getDataList()
              this.$message.success('取消预留成功')
            } else {
              this.$message.error('取消预留失败')
            }
          })
          .catch(() => {
            console.log('取消预留时，发生错误')
          })
      } else {
        this.$message.warning('请选择需要取消预留的库存')
      }
    },
    cancelReservedHandle1() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        const joinStockOrderList = []
        let joinMultiple
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          joinMultiple = this.joinMultipleSelection[i]
          joinStockOrderList.push({
            source: 'erp',
            id: joinMultiple.id,
            operationTaskId: joinMultiple.operationTaskId,
            stockId: joinMultiple.stockId,
            reservedQuantity: joinMultiple.reservedQuantity,
            partNo: joinMultiple.partNo,
            partName: joinMultiple.partName,
            warehouseNo: joinMultiple.warehouseNo,
            warehouseName: joinMultiple.warehouseName,
            locationNo: joinMultiple.locationNo,
            locationName: joinMultiple.locationName,
            partUnit: joinMultiple.partUnit,
            systemNo: joinMultiple.systemNo,
            partBatchNo: joinMultiple.partBatchNo,
            status: '02confirmed',
            type: '工单预留'
          })
        }
        cancelReserve(joinStockOrderList)
          .then((response) => {
            const resCode = response.data.code
            if (resCode === 0) {
              this.$refs.joinStockTable.getDataList()
              this.$message.success('取消预留成功')
            } else {
              this.$message.error('取消预留失败')
            }
          })
          .catch(() => {
            console.log('取消预留时，发生错误')
          })
      } else {
        this.$message.warning('请选择需要取消预留的库存')
      }
    },
    // 确认
    confirmHandle() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          if (this.joinMultipleSelection[i].status === '02confirmed') {
            this.$message.warning('请选择待确认的预留库存')
            return
          }
        }
        changeState(this.joinMultipleSelection, 'CONFIRM').then((response) => {
          this.$refs.joinStockTable.getDataList()
          this.$message.success('确认成功')
          this.refreshCustomerOrder()
        })
      } else {
        this.$message.warning('请选择需要确认的库存')
      }
    },
    // 取消确认
    cancelConfirmHandle() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          if (this.joinMultipleSelection[i].status === '01unconfirmed') {
            this.$message.warning('请选择已确认的预留库存')
            return
          }
        }
        changeState(this.joinMultipleSelection, 'CANCEL').then((response) => {
          this.$refs.joinStockTable.getDataList()
          this.$message.success('取消确认成功')
          this.refreshCustomerOrder()
        })
      } else {
        this.$message.warning('请选择需要取消确认的库存')
      }
    },
    // 刷新客户订单的已预留数量
    refreshCustomerOrder() {
      if (this.currentOperationTask && this.currentOperationTask.id) {
        updateOperationTaskId(this.currentOperationTask.id).then((response) => {
          this.$set(
            this.currentOperationTask,
            'reservedQuantity',
            response.data.data
          )
        })
      }
    },
    handleReserved() {
      this.$refs.joinStockTable.getDataList()
    },
    pickScm() {
      if (this.currentOperationTask) {
        matPick(this.currentOperationTask.id).then((response) => {
          this.$message.success('SCM领料成功')
        })
      } else {
        this.$message.warning('请先选择一条工单')
      }
    }
  }
}
</script>
